#include "stdafx.h"
#include "tga.h"

int Screen_width=0;
int Screen_height=0;

#define PIXEL_X_TO_MOUSEINPUT(x) (LONG)(((float)x/(float)Screen_width)*65535)
#define PIXEL_Y_TO_MOUSEINPUT(x) (LONG)(((float)x/(float)Screen_height)*65535)

// Dimensions of the board, in pixels
#define BOARD_WIDTH	 (300)
#define BOARD_HEIGHT (300)

// Upper left pixel (in screen space) of the drawing board
int BOARD_X1=0;
int BOARD_Y1=0;

// Some globals
char App_args[50][255];
int Num_args=0;

// Gathers all arguments passed in from the command line, should be called at the first part of the program
void GatherArgs (char *str)
{
	int i,t,curarg=1;
	int len=(int)strlen(str);
	bool gotquote = false;

	for (t=0,i=0;i<len;i++)
	{
		if(t==0)
		{
			if(str[i]=='\"')
			{
				gotquote = true;
				continue;
			}
		}
		if(gotquote)
		{
			if (str[i]!='\"' && str[i]!=0)
				App_args[curarg][t++]=str[i];
			else
			{
				App_args[curarg][t]=0;
				t=0;
				gotquote = false;
				curarg++;
			}
		}
		else
		{
			if (str[i]!=' ' && str[i]!=0)
				App_args[curarg][t++]=str[i];
			else
			{
				App_args[curarg][t]=0;
				t=0;
				curarg++;
			}
		}
	}

	App_args[curarg][t]=0;
	curarg++;
	Num_args=curarg;
}

// Returns index of argument sought, or 0 if not found
int FindArg (char *which)
{
	int i;

	for (i=1;i<=Num_args;i++)
		if (stricmp (which,App_args[i])==0)
			return (i);

	return (0);
}



// Given a x,y screen space coordinate, fools the system into thinking the user has clicked the mouse button at that location
inline void SendCommandToRich (int x,int y)
{
	INPUT mouse_inputs[1];

	memset (&mouse_inputs[0],0,sizeof(INPUT));
	mouse_inputs[0].type=INPUT_MOUSE;
	mouse_inputs[0].mi.dx=PIXEL_X_TO_MOUSEINPUT(x);
	mouse_inputs[0].mi.dy=PIXEL_Y_TO_MOUSEINPUT(y);
	mouse_inputs[0].mi.dwFlags=MOUSEEVENTF_MOVE|MOUSEEVENTF_ABSOLUTE|MOUSEEVENTF_LEFTDOWN;
	UINT val=SendInput (1,mouse_inputs,sizeof(INPUT));
	
	Sleep (1);
	memset (&mouse_inputs[0],0,sizeof(INPUT));
	mouse_inputs[0].type=INPUT_MOUSE;
	mouse_inputs[0].mi.dx=PIXEL_X_TO_MOUSEINPUT(x);
	mouse_inputs[0].mi.dy=PIXEL_Y_TO_MOUSEINPUT(y);
	mouse_inputs[0].mi.dwFlags=MOUSEEVENTF_MOVE|MOUSEEVENTF_ABSOLUTE|MOUSEEVENTF_LEFTUP;
	val=SendInput (1,mouse_inputs,sizeof(INPUT));
	Sleep (1);
}

// Given line screen space coordinates, fools the system into thinking the user wants to draw a line there
inline void SendLineCommandToRich (int x1,int y1,int x2,int y2)
{
	INPUT mouse_inputs[1];

	memset (&mouse_inputs[0],0,sizeof(INPUT));
	mouse_inputs[0].type=INPUT_MOUSE;
	mouse_inputs[0].mi.dx=PIXEL_X_TO_MOUSEINPUT(x1);
	mouse_inputs[0].mi.dy=PIXEL_Y_TO_MOUSEINPUT(y1);
	mouse_inputs[0].mi.dwFlags=MOUSEEVENTF_MOVE|MOUSEEVENTF_ABSOLUTE|MOUSEEVENTF_LEFTDOWN;
	UINT val=SendInput (1,mouse_inputs,sizeof(INPUT));

	Sleep (1);
	memset (&mouse_inputs[0],0,sizeof(INPUT));
	mouse_inputs[0].type=INPUT_MOUSE;
	mouse_inputs[0].mi.dx=PIXEL_X_TO_MOUSEINPUT(x2);
	mouse_inputs[0].mi.dy=PIXEL_Y_TO_MOUSEINPUT(y2);
	mouse_inputs[0].mi.dwFlags=MOUSEEVENTF_MOVE|MOUSEEVENTF_ABSOLUTE|MOUSEEVENTF_LEFTUP;
	val=SendInput (1,mouse_inputs,sizeof(INPUT));
	Sleep (1);
}

// Clicks the brush tool on Drawrings
void ClickBrush ()
{
	SendCommandToRich (840,350);
}


// Clicks the Pen tool on Drawrings
void ClickPen ()
{
	SendCommandToRich (840,200);
}

// Cylces the Pen tool on Drawrings
void ClickPenUp ()
{
	SendCommandToRich (880,197);
}

// Clicks on the black color
void ClickBlack ()
{
	SendCommandToRich (800,435);
}

// Clicks on the white color
void ClickWhite ()
{
	SendCommandToRich (784,435);
}

// Clicks on the grey color
void ClickGrey ()
{
	SendCommandToRich (784,460);
}

// Tells the drawring app to change to the requested r,g,b color
void ChangeColor (int r,int g,int b)
{
	r=min(255,r);
	r=max(0,r);
	
	g=min(255,g);
	g=max(0,g);
	
	b=min(255,b);
	b=max(0,b);
	

	r/=5;
	g/=5;
	b/=5;
	
	SendCommandToRich (365+BOARD_X1+r,172+BOARD_Y1);
	SendCommandToRich (365+BOARD_X1+g,172+BOARD_Y1+40);
	SendCommandToRich (365+BOARD_X1+b,172+BOARD_Y1+80);
}

// Retrieves the upper left corner of the Drawring page
// We do this by looking for a row of solid black pixels surrounded by white pixels to either side of that row
bool GetUpperLeftCornerOfDrawring(int *x,int *y)
{
	HDC hdc=CreateDC(TEXT("DISPLAY"),NULL,NULL,NULL);

	int running_count=0,startx,starty;
	for (int i=200;i<400;i++)
	{
		running_count=0;
		for (int t=50;t<800;t++)
		{
			COLORREF l1=GetPixel(hdc,t,i-1); // Line 1 (abov)
			COLORREF l2=GetPixel(hdc,t,i);   // Line 2 (target line)
			COLORREF l3=GetPixel(hdc,t,i+1);   // Line 3 (below)

			if (l2==0x00999999 && l1==0x00FFFFFF)
			{
				if (running_count==0)
				{
					startx=t;
					starty=i;
				}
				
				running_count++;
				if (running_count>=280)
				{
					// Found a 100 pixels in a row, probably the start of the Drawring rectangle
					*x=startx;
					*y=starty+2;
					return true;
				}
			}
			else
			{
				running_count=0;
			}
		}
	}
	return false;
}
	

// Main entry point of program
int _tmain(int argc, _TCHAR* argv[])
{
	UINT count=0;

	// Grab the arguments coming into the program
	char args[255*50]={0};
	for (int i=0;i<argc;i++)
	{
		strcat(args,argv[i]);
		if (i!=argc-1)
			strcat(args," ");
	}

	GatherArgs(args);

	// Minimize this window
	ShowWindow (GetConsoleWindow(),SW_MINIMIZE);

	// Give the user 5 seconds to click to the window
	Sleep (5000);

	// Get screen dimensions
	Screen_width=GetSystemMetrics(SM_CXSCREEN);
	Screen_height=GetSystemMetrics(SM_CYSCREEN);

	// Find the start of the drawring board
	int found_start=GetUpperLeftCornerOfDrawring(&BOARD_X1,&BOARD_Y1);
	if (!found_start)
	{
		printf ("Couldn't find upper left corner of Drawring page...did you make that window active and to the front?\n");
		exit(-1);
	}

	printf ("Found start of Drawring board at %d , %d \n",BOARD_X1,BOARD_Y1);

	/* // This commented code is for grabing the x,y values of the mouse...its debug code and isn't used anymore
	while (1)
	{
		count++;
		if ((count%20000000)==0)
		{
			count=1;
			POINT pnt;
			GetCursorPos(&pnt);
			printf ("x=%d y=%d\n",pnt.x,pnt.y);
		}
	
	}*/
	

	// Check for required image argument
	TGAImg img,img2;
	if (FindArg("-image")==0)
	{
		printf ("You must at least supply the name of a image, fool!\n");
		exit(-1);
	}
	
	//Load image
	int image_index=FindArg("-image")+1;
	int success=img.Load(App_args[image_index]);

	// Grab screen size
	if (FindArg("-1280x800"))
	{
		Screen_width=1280;
		Screen_height=800;
	}

	// Grab some bools!
	bool ignore_white=FindArg("-ignorewhite")?true:false;
	bool ignore_black=FindArg("-ignoreblack")?true:false;
	bool compress_color=FindArg("-compresscolor")?true:false;
	bool jig_from_center=FindArg("-jigfromcenter")?true:false;
	bool use_lines=FindArg("-uselines")?true:false;

	// Now get the various factors
	int stepx=1,stepy=1;
	if (FindArg("-stepx"))
		stepx=atoi(App_args[FindArg("-stepx")+1]);

	if (FindArg("-stepy"))
		stepy=atoi(App_args[FindArg("-stepy")+1]);

	int random_color_factor=1;
	if (FindArg("-randomcolorfactor"))
		random_color_factor=atoi(App_args[FindArg("-randomcolorfactor")+1]);

	int jig_factor=1;
	if (FindArg("-jigfactor"))
		jig_factor=atoi(App_args[FindArg("-jigfactor")+1]);

	float crossfade=0.0f;
	if (FindArg("-crossfade"))
		crossfade=(float)atof(App_args[FindArg("-crossfade")+1]);

	// Check to see if we're using image2, if so, load it!
	int image2_index=FindArg("-image2")+1;
	if (image2_index>1)
	{
		int success2=img2.Load(App_args[image2_index]);
		if (success2!=IMG_OK)
		{	
			printf ("Couldn't load image2!\n");
			exit (-1);
		}
	}
		
	int bpp=img.GetBPP()/8;
	int width=img.GetWidth();
	int height=img.GetHeight();
	unsigned int *src_data=(unsigned int *)img.GetImg();
	bool suspend_mode=false;
			

	// Iterate over the drawring board in a column x row fashion, stepping the desired amount each time
	for (int i=0;i<BOARD_HEIGHT;i+=stepy)
	{
		for (int t=0;t<BOARD_WIDTH;t+=stepx)
		{
			// Check for keys that can interrupt us!
			SHORT ekey=GetAsyncKeyState(VK_ESCAPE);
			if (ekey & (1<<15))
				exit(-1);

			ekey=GetAsyncKeyState(VK_SUBTRACT);
			if (ekey & (1<<15))
				suspend_mode=true;

			ekey=GetAsyncKeyState(VK_ADD);
				if (ekey & (1<<15))
					suspend_mode=false;

			if (suspend_mode)
			{
				t-=stepx; // Dumb & lazy way to do this, but I'm tired
				continue;
			}

			// Get normalized value (0.0 to 1.0) based on where we are in the destination Drawring image
			float norm_x=(float)t/(float)BOARD_WIDTH;
			float norm_y=(float)i/(float)BOARD_HEIGHT;

			// See how far away we are from the center of the board - gives -1 to 1.0 range
			float center_norm_x=-1.0f+((float)t/(float)(BOARD_WIDTH/2));
			float center_norm_y=-1.0f+((float)i/(float)(BOARD_HEIGHT/2));
			
			// Grab source pixel from image
			int src_x=(int)(norm_x*(float)width);
			int src_y=(int)(norm_y*(float)height);
			
			unsigned int pixel=src_data[(src_y*width)+src_x];
			
			int r=(pixel>>0) & 0xFF;
			int g=(pixel>>8) & 0xFF;
			int b=(pixel>>16) & 0xFF;

			if (crossfade>0.0f)
			{
				// If we're crossfading, grab the x,y normalized pixel out of image2
				int width2=img2.GetWidth();
				int height2=img2.GetHeight();
				unsigned int *src2_data=(unsigned int *)img2.GetImg();

				int src2_x=(int)(norm_x*(float)width2);
				int src2_y=(int)(norm_y*(float)height2);
				unsigned int pixel2=src2_data[(src2_y*width2)+src2_x];

				int r2=(pixel2>>0) & 0xFF;
				int g2=(pixel2>>8) & 0xFF;
				int b2=(pixel2>>16) & 0xFF;

				// Now blend image1 and image2 pixel based on the crossfade ratio
				r=(int)(((float)(r2*crossfade))+((float)(r*(1.0f-crossfade))));
				g=(int)(((float)(g2*crossfade))+((float)(g*(1.0f-crossfade))));
				b=(int)(((float)(b2*crossfade))+((float)(b*(1.0f-crossfade))));

			}
									
			// Ignore white?
			if (ignore_white && (r>=250 && g>=250 && b>=250))
				continue;

			// Ignore black?
			if (ignore_black && (r<5 && g<5 && b<5))
				continue;
	
			// Turns our colors into quantized goodness.  Gets rid of the 3 least-signifcant-bits.
			if (compress_color)
			{
				r>>=3; r<<=3;
				g>>=3; g<<=3;
				b>>=3; b<<=3;
			}
			
			// Randomize color if desired
			if (random_color_factor>1)
				ChangeColor(r-random_color_factor+(rand()%random_color_factor*2),g-random_color_factor+(rand()%random_color_factor*2),b-random_color_factor+(rand()%random_color_factor*2));
			else
				ChangeColor(r,g,b);

			// Calculate jig randomness
			int x_jig,y_jig;
			x_jig=jig_factor-((rand()%(jig_factor*2)));
			y_jig=jig_factor-((rand()%(jig_factor*2)));
			
			// If jigging from center, make the center more jiggly, and the edges less so
			if (jig_from_center)
			{
				x_jig=(int)((float)x_jig*center_norm_x);
				y_jig=(int)((float)y_jig*center_norm_y);
			}

			if (jig_factor==1)
			{
				x_jig=0;
				y_jig=0;
			}

			// Now, falsify the mouse clicks!
			if (use_lines)  
				SendLineCommandToRich (BOARD_X1+t,BOARD_Y1+i,BOARD_X1+t+x_jig,BOARD_Y1+i+y_jig);
			else
				SendCommandToRich (BOARD_X1+t+x_jig,BOARD_Y1+i+y_jig);
		}	
	}

	printf ("Have a day!\n");
}

